@@ -30,6 +30,7 @@
 #include "commands.h"
 #include "ranges.h"
 #include "parse-util.h"
+#include "gutils.h"
 #include "gui-util.h"
 #include "sheet-autofill.h"
 #include <gsf/gsf-impl-utils.h>
@@ -58,7 +59,7 @@ struct _GnmItemCursor {
 	GnmCellPos last_tip_pos;
 
 	GnmItemCursorStyle style;
-	int      state;
+	guint    ant_state;
 	guint    animation_timer;
 
 	/*
@@ -80,7 +81,6 @@ struct _GnmItemCursor {
 	int drag_button;
 	guint drag_button_state;
 
-	gboolean visible;
 	gboolean use_color;
 	gboolean auto_fill_handle_at_top;
 	gboolean auto_fill_handle_at_left;
@@ -124,6 +124,11 @@ ic_reload_style (GnmItemCursor *ic)
 	gtk_style_context_get_background_color (context, state,
 						&ic->ant_background_color);
 	gtk_style_context_restore (context);
+	/*
+	 * Ensure we don't use transparency to avoid compositing issues
+	 * when redrawing the ants in the timer callback.
+	 */
+	ic->ant_color.alpha = ic->ant_background_color.alpha = 1.;
 
 	gtk_style_context_save (context);
 	gtk_style_context_add_class (context, "drag");
@@ -144,9 +149,38 @@ static int
 cb_item_cursor_animation (GnmItemCursor *ic)
 {
 	GocItem *item = GOC_ITEM (ic);
+	static int use_fallback = -1;
+
+	ic->ant_state++;
+
+	if (use_fallback != 1) {
+		GdkWindow *w = gtk_widget_get_window (GTK_WIDGET (item->canvas));
+		cairo_t *cr = gdk_cairo_create (w);
+
+		if (use_fallback == -1) {
+			/*
+			 * Under some themes we get a (0,0,0,0) clip and
+			 * our drawing does nothing.  Detect this and
+			 * fall back to full invalidation.
+			 */
+			double cx0, cy0, cx1, cy1;
+			cairo_clip_extents (cr, &cx0, &cy0, &cx1, &cy1);
+			use_fallback = (cx0 == 0 && cy0 == 0 && cx1 == 0 && cy1 == 0);
+			if (use_fallback == 1 && gnm_debug_flag ("ant"))
+				g_printerr ("Using ant-ing fallback\n");
+		}
+
+		if (use_fallback == 0)
+			goc_item_draw (item, cr);
+
+		cairo_destroy (cr);
+	}
+
+	if (use_fallback == 1) {
+		if (ic->ant_state % 2 == 1)
+			goc_item_invalidate (item);
+	}
 
-	ic->state = !ic->state;
-	goc_item_invalidate (item);
 	return TRUE;
 }
 
@@ -175,7 +209,7 @@ item_cursor_realize (GocItem *item)
 	if (ic->style == GNM_ITEM_CURSOR_ANTED) {
 		g_return_if_fail (ic->animation_timer == 0);
 		ic->animation_timer = g_timeout_add (
-			150, (GSourceFunc) cb_item_cursor_animation,
+			75, (GSourceFunc) cb_item_cursor_animation,
 			ic);
 	}
 }
@@ -239,6 +273,7 @@ item_cursor_draw (GocItem const *item, cairo_t *cr)
 	gboolean draw_center, draw_external, draw_internal, draw_xor;
 	double scale = item->canvas->pixels_per_unit;
 	GdkRGBA *fore = NULL, *back = NULL;
+	double phase0 = 0;
 
 #if 0
 	g_printerr ("draw[%d] %lx,%lx %lx,%lx\n",
@@ -248,7 +283,7 @@ item_cursor_draw (GocItem const *item, cairo_t *cr)
 		    ic->outline.x2,
 		    ic->outline.y2);
 #endif
-	if (!ic->visible || !ic->pos_initialized)
+	if (!goc_item_is_visible (&ic->canvas_item) || !ic->pos_initialized)
 		return;
 
 	/* we need to use canvas coordinates in goc_canvas_c2w, hence the divisions by scale. */
@@ -329,13 +364,11 @@ item_cursor_draw (GocItem const *item, cairo_t *cr)
 	case GNM_ITEM_CURSOR_ANTED:
 		draw_center   = TRUE;
 		draw_thick    = 2;
+		draw_xor = FALSE;
 		fore = &ic->ant_color;
 		back = &ic->ant_background_color;
-		if (ic->state) {
-			GdkRGBA *swap = fore;
-			fore = back;
-			back = swap;
-		}
+		phase0 = (~ic->ant_state & 3) * 0.25;
+		break;
 	}
 
 	if (ic->use_color) {
@@ -446,18 +479,21 @@ item_cursor_draw (GocItem const *item, cairo_t *cr)
 
 	if (draw_center) {
 		double dashes[2];
+		double phase1 = fmod (phase0 + 0.5, 1);
 
 		/* Stay in the boundary */
 		x0 += (draw_thick / 2.0);
 		y0 += (draw_thick / 2.0);
 
-		dashes[0] = dashes[1] = draw_stippled;
-		cairo_set_dash (cr, dashes, 2, 0.);
 		cairo_set_line_width (cr, draw_thick);
-		gdk_cairo_set_source_rgba (cr, back);
 		cairo_rectangle (cr, x0, y0, abs (x1 - x0), abs (y1 - y0));
+		dashes[0] = dashes[1] = draw_stippled;
+
+		cairo_set_dash (cr, dashes, 2, phase0 * 2 * draw_stippled);
+		gdk_cairo_set_source_rgba (cr, back);
 		cairo_stroke_preserve (cr);
-		cairo_set_dash (cr, dashes, 2, dashes[1]);
+
+		cairo_set_dash (cr, dashes, 2, phase1 * 2 * draw_stippled);
 		gdk_cairo_set_source_rgba (cr, fore);
 		cairo_stroke (cr);
 	}
@@ -511,7 +547,8 @@ item_cursor_distance (GocItem *item, double x, double y,
 	 * 2) when animated
 	 * 3) while a guru is up
 	 */
-	if (!ic->visible || ic->style == GNM_ITEM_CURSOR_ANTED ||
+	if (!goc_item_is_visible (item) ||
+	    ic->style == GNM_ITEM_CURSOR_ANTED ||
 	    wbc_gtk_get_guru (scg_wbcg (ic->scg)) != NULL)
 		return DBL_MAX;
 
@@ -608,7 +645,7 @@ item_cursor_selection_motion (GocItem *item, double x_, double y_)
 	int style, button;
 	gint64 x = x_ * canvas->pixels_per_unit, y = y_ * canvas->pixels_per_unit;
 	GnmItemCursor *special_cursor;
-	GdkEventMotion *event = (GdkEventMotion *) goc_canvas_get_cur_event (item->canvas);
+	GdkEvent *event = goc_canvas_get_cur_event (item->canvas);
 
 	if (ic->drag_button < 0) {
 		item_cursor_set_cursor (canvas, ic, x, y);
@@ -626,7 +663,7 @@ item_cursor_selection_motion (GocItem *item, double x_, double y_)
 
 	button = ic->drag_button;
 	ic->drag_button = -1;
-	gnm_simple_canvas_ungrab (item, event->time);
+	gnm_simple_canvas_ungrab (item, gdk_event_get_time (event));
 
 	scg_special_cursor_start (ic->scg, style, button);
 	special_cursor = pane->cursor.special;
@@ -673,7 +710,7 @@ item_cursor_selection_motion (GocItem *item, double x_, double y_)
 
 	gnm_simple_canvas_grab (GOC_ITEM (special_cursor),
 		GDK_POINTER_MOTION_MASK | GDK_BUTTON_RELEASE_MASK | GDK_BUTTON_PRESS_MASK,
-		NULL, event->time);
+		NULL, gdk_event_get_time (event));
 	gnm_pane_slide_init (pane);
 
 	goc_item_bounds_changed (GOC_ITEM (ic));
@@ -777,7 +814,7 @@ context_menu_hander (GnumericPopupMenuElement const *element,
 }
 
 static void
-item_cursor_popup_menu (GnmItemCursor *ic, GdkEventButton *event)
+item_cursor_popup_menu (GnmItemCursor *ic, GdkEvent *event)
 {
 	static GnumericPopupMenuElement const popup_elements[] = {
 		{ N_("_Move"),		NULL,
@@ -816,7 +853,7 @@ item_cursor_popup_menu (GnmItemCursor *ic, GdkEventButton *event)
 }
 
 static void
-item_cursor_do_drop (GnmItemCursor *ic, GdkEventButton *event)
+item_cursor_do_drop (GnmItemCursor *ic, GdkEvent *event)
 {
 	/* Only do the operation if something moved */
 	SheetView const *sv = scg_view (ic->scg);
@@ -828,10 +865,10 @@ item_cursor_do_drop (GnmItemCursor *ic, GdkEventButton *event)
 		return;
 	}
 
-	if (event->button == 3)
+	if (event->button.button == 3)
 		item_cursor_popup_menu (ic, event);
 	else
-		item_cursor_do_action (ic, (event->state & GDK_CONTROL_MASK)
+		item_cursor_do_action (ic, (event->button.state & GDK_CONTROL_MASK)
 				       ? ACTION_COPY_CELLS
 				       : ACTION_MOVE_CELLS);
 }
@@ -839,16 +876,7 @@ item_cursor_do_drop (GnmItemCursor *ic, GdkEventButton *event)
 void
 gnm_item_cursor_set_visibility (GnmItemCursor *ic, gboolean visible)
 {
-	g_return_if_fail (GNM_IS_ITEM_CURSOR (ic));
-
-	if (ic->visible == visible)
-		return;
-
-	ic->visible = visible;
-	if (visible)
-		goc_item_show (GOC_ITEM (ic));
-	else
-		goc_item_hide (GOC_ITEM (ic));
+	goc_item_set_visible (GOC_ITEM (ic), visible);
 }
 
 static void
@@ -1033,7 +1061,8 @@ item_cursor_button_pressed (GocItem *item, int button, double x_, double y_)
 {
 	GnmItemCursor *ic = GNM_ITEM_CURSOR (item);
 	gint64 x = x_ * item->canvas->pixels_per_unit, y = y_ * item->canvas->pixels_per_unit;
-	GdkEventButton *event = (GdkEventButton *) goc_canvas_get_cur_event (item->canvas);
+	GdkEvent *event = goc_canvas_get_cur_event (item->canvas);
+	GdkEventButton *bevent = &event->button;
 	if (ic->style == GNM_ITEM_CURSOR_EXPR_RANGE)
 		return FALSE;
 
@@ -1077,10 +1106,10 @@ item_cursor_button_pressed (GocItem *item, int button, double x_, double y_)
 								     _("Drag to move"));
 
 			ic->drag_button = button;
-			ic->drag_button_state = event->state;
+			ic->drag_button_state = bevent->state;
 			gnm_simple_canvas_grab (item,
 				GDK_POINTER_MOTION_MASK | GDK_BUTTON_RELEASE_MASK | GDK_BUTTON_PRESS_MASK,
-				NULL, event->time);
+				NULL, gdk_event_get_time (event));
 		} else
 			scg_context_menu (ic->scg, event, FALSE, FALSE);
 		return TRUE;
@@ -1103,7 +1132,7 @@ static gboolean
 item_cursor_button2_pressed (GocItem *item, int button, double x_, double y_)
 {
 	GnmItemCursor *ic = GNM_ITEM_CURSOR (item);
-	GdkEventButton *event = (GdkEventButton *) goc_canvas_get_cur_event (item->canvas);
+	GdkEvent *event = goc_canvas_get_cur_event (item->canvas);
 
 	switch (ic->style) {
 
@@ -1116,7 +1145,7 @@ item_cursor_button2_pressed (GocItem *item, int button, double x_, double y_)
 			return TRUE;
 
 		ic->drag_button = -1;
-		gnm_simple_canvas_ungrab (item, event->time);
+		gnm_simple_canvas_ungrab (item, gdk_event_get_time (event));
 
 		if (sheet_is_region_empty (sheet, &ic->pos))
 			return TRUE;
@@ -1134,7 +1163,7 @@ item_cursor_button2_pressed (GocItem *item, int button, double x_, double y_)
 		 * find the boundary of the region being filled.
 		 */
 
-		if (event->state & GDK_MOD1_MASK) {
+		if (event->button.state & GDK_MOD1_MASK) {
 			int template_col = ic->pos.end.col + 1;
 			int template_row = ic->pos.start.row - 1;
 			int boundary_col_for_target;
@@ -1309,7 +1338,7 @@ static gboolean
 item_cursor_button_released (GocItem *item, int button, G_GNUC_UNUSED double x, G_GNUC_UNUSED double y)
 {
 	GnmItemCursor *ic = GNM_ITEM_CURSOR (item);
-	GdkEventButton *event = (GdkEventButton *) goc_canvas_get_cur_event (item->canvas);
+	GdkEvent *event = goc_canvas_get_cur_event (item->canvas);
 	WBCGtk *wbcg = scg_wbcg (ic->scg);
 
 	if (ic->style == GNM_ITEM_CURSOR_EXPR_RANGE)
@@ -1331,7 +1360,7 @@ item_cursor_button_released (GocItem *item, int button, G_GNUC_UNUSED double x,
 
 		/* Double clicks may have already released the drag prep */
 		if (ic->drag_button >= 0) {
-			gnm_simple_canvas_ungrab (item, event->time);
+			gnm_simple_canvas_ungrab (item, gdk_event_get_time (event));
 			ic->drag_button = -1;
 		}
 		go_cmd_context_progress_message_set (GO_CMD_CONTEXT (wbcg),
@@ -1343,7 +1372,7 @@ item_cursor_button_released (GocItem *item, int button, G_GNUC_UNUSED double x,
 			return TRUE;
 
 		gnm_pane_slide_stop (GNM_PANE (item->canvas));
-		gnm_simple_canvas_ungrab (item, event->time);
+		gnm_simple_canvas_ungrab (item, gdk_event_get_time (event));
 		item_cursor_do_drop (ic, event);
 
 		go_cmd_context_progress_message_set (GO_CMD_CONTEXT (wbcg),
@@ -1359,7 +1388,7 @@ item_cursor_button_released (GocItem *item, int button, G_GNUC_UNUSED double x,
 		SheetControlGUI *scg = ic->scg;
 
 		gnm_pane_slide_stop (GNM_PANE (item->canvas));
-		gnm_simple_canvas_ungrab (item, event->time);
+		gnm_simple_canvas_ungrab (item, gdk_event_get_time (event));
 
 		cmd_autofill (scg_wbc (scg), scg_sheet (scg), default_increment,
 			      ic->pos.start.col, ic->pos.start.row,
@@ -1496,10 +1525,9 @@ gnm_item_cursor_init (GnmItemCursor *ic)
 	ic->last_y = 0;
 
 	ic->style = GNM_ITEM_CURSOR_SELECTION;
-	ic->state = 0;
+	ic->ant_state = 0;
 	ic->animation_timer = 0;
 
-	ic->visible = TRUE;
 	ic->auto_fill_handle_at_top = FALSE;
 	ic->auto_fill_handle_at_left = FALSE;
 	ic->drag_button = -1;
