@@ -7,6 +7,9 @@
 #define ML_UNDERFLOW (GNM_EPSILON * GNM_EPSILON)
 #define ML_ERROR(cause) do { } while(0)
 
+static int qgammaf (gnm_float x, GnmQuad *mant, int *exp2);
+
+
 /* Compute  gnm_log(gamma(a+1))  accurately also for small a (0 < a < 0.5). */
 gnm_float lgamma1p (gnm_float a)
 {
@@ -460,11 +463,14 @@ gnm_float gnm_lbeta(gnm_float a, gnm_float b)
 gnm_float
 gnm_gamma (gnm_float x)
 {
-	if (gnm_abs (x) < 0.5) {
-		gnm_float a = gnm_exp (gnm_lgamma (x));
-		return (x > 0) ? a : -a;
-	} else
-		return gnm_fact (x - 1);
+	GnmQuad r;
+	int e;
+
+	switch (qgammaf (x, &r, &e)) {
+	case 0: return ldexp (gnm_quad_value (&r), e);
+	case 1: return gnm_pinf;
+	default: return gnm_nan;
+	}
 }
 
 /* ------------------------------------------------------------------------- */
@@ -490,6 +496,33 @@ gnm_fact (gnm_float x)
 
 /* ------------------------------------------------------------------------- */
 
+/* 0: ok, 1: overflow, 2: nan */
+static int
+qbetaf (gnm_float a, gnm_float b, GnmQuad *mant, int *exp2)
+{
+	GnmQuad ma, mb, mab;
+	int ea, eb, eab;
+	gnm_float ab = a + b;
+
+	if (gnm_isnan (ab) ||
+	    (a <= 0 && a == gnm_floor (a)) ||
+	    (b <= 0 && b == gnm_floor (b)) ||
+	    (ab <= 0 && ab == gnm_floor (ab)))
+		return 2;
+
+	if (!qgammaf (a, &ma, &ea) &&
+	    !qgammaf (b, &mb, &eb) &&
+	    !qgammaf (ab, &mab, &eab)) {
+		void *state = gnm_quad_start ();
+		gnm_quad_mul (&ma, &ma, &mb);
+		gnm_quad_div (mant, &ma, &mab);
+		gnm_quad_end (state);
+		*exp2 = ea + eb - eab;
+		return 0;
+	} else
+		return 1;
+}
+
 /**
  * gnm_beta:
  * @a: a number
@@ -500,10 +533,14 @@ gnm_fact (gnm_float x)
 gnm_float
 gnm_beta (gnm_float a, gnm_float b)
 {
-	int sign;
-	gnm_float absres = gnm_exp (gnm_lbeta3 (a, b, &sign));
+	GnmQuad r;
+	int e;
 
-	return sign == -1 ? -absres : absres;
+	switch (qbetaf (a, b, &r, &e)) {
+	case 0: return ldexp (gnm_quad_value (&r), e);
+	case 1: return gnm_pinf;
+	default: return gnm_nan;
+	}
 }
 
 /**
@@ -523,21 +560,29 @@ gnm_lbeta3 (gnm_float a, gnm_float b, int *sign)
 	int sign_a, sign_b, sign_ab;
 	gnm_float ab = a + b;
 	gnm_float res_a, res_b, res_ab;
+	GnmQuad r;
+	int e;
 
-	*sign = 1;
-	if (a > 0 && b > 0)
-		return gnm_lbeta (a, b);
-
-#ifdef IEEE_754
-	if (gnm_isnan(ab))
-		return ab;
-#endif
-
-	if ((a <= 0 && a == gnm_floor (a)) ||
-	    (b <= 0 && b == gnm_floor (b)) ||
-	    (ab <= 0 && ab == gnm_floor (ab)))
+	switch (qbetaf (a, b, &r, &e)) {
+	case 0: {
+		gnm_float m = gnm_quad_value (&r);
+		*sign = (m >= 0 ? +1 : -1);
+		return gnm_log (gnm_abs (m)) + e * M_LN2gnum;
+	}
+	case 1:
+		/* Overflow */
+		break;
+	default:
+		*sign = 1;
 		return gnm_nan;
+	}
 
+	if (a > 0 && b > 0) {
+		*sign = 1;
+		return gnm_lbeta (a, b);
+	}
+
+	/* This is awful */
 	res_a = gnm_lgamma_r (a, &sign_a);
 	res_b = gnm_lgamma_r (b, &sign_b);
 	res_ab = gnm_lgamma_r (ab, &sign_ab);
@@ -547,29 +592,37 @@ gnm_lbeta3 (gnm_float a, gnm_float b, int *sign)
 }
 
 /* ------------------------------------------------------------------------- */
-
+/*
+ * This computes the E(x) such that
+ *
+ *   Gamma(x) = sqrt(2Pi) * x^(x-1/2) * exp(-x) * E(x)
+ *
+ * x should be >20 and the result is, roughly, 1+1/(12x).
+ */
 static void
 gamma_error_factor (GnmQuad *res, const GnmQuad *x)
 {
 	gnm_float num[] = {
-		1.,
-		1.,
-		-139.,
-		-571.,
-		163879.,
-		5246819.,
-		-534703531.,
-		-4483131259.
+		GNM_const(1.),
+		GNM_const(1.),
+		GNM_const(-139.),
+		GNM_const(-571.),
+		GNM_const(163879.),
+		GNM_const(5246819.),
+		GNM_const(-534703531.),
+		GNM_const(-4483131259.),
+		GNM_const(432261921612371.)
 	};
 	gnm_float den[] = {
-		12.,
-		288.,
-		51840.,
-		2488320.,
-		209018880.,
-		75246796800.,
-		902961561600.,
-		86684309913600.
+		GNM_const(12.),
+		GNM_const(288.),
+		GNM_const(51840.),
+		GNM_const(2488320.),
+		GNM_const(209018880.),
+		GNM_const(75246796800.),
+		GNM_const(902961561600.),
+		GNM_const(86684309913600.),
+		GNM_const(514904800886784000.)
 	};
 	GnmQuad zn;
 	int i;
@@ -593,7 +646,7 @@ gamma_error_factor (GnmQuad *res, const GnmQuad *x)
 static void
 pochhammer_small_n (gnm_float x, gnm_float n, GnmQuad *res)
 {
-	GnmQuad qx, qn, qr, qs, qone, f1, f2, f3, f4, f5;
+	GnmQuad qx, qn, qr, qs, f1, f2, f3, f4, f5;
 	gnm_float r;
 	gboolean debug = FALSE;
 
@@ -623,15 +676,13 @@ pochhammer_small_n (gnm_float x, gnm_float n, GnmQuad *res)
 
 	gnm_quad_add (&qs, &qx, &qn);
 
-	gnm_quad_init (&qone, 1);
-
 	/* exp(x*log1pmx(n/x)) */
 	gnm_quad_mul12 (&f1, log1pmx (r), x);  /* sub-opt */
 	gnm_quad_exp (&f1, NULL, &f1);
 	if (debug) g_printerr ("f1=%.20g\n", gnm_quad_value (&f1));
 
 	/* sqrt(1+n/x) */
-	gnm_quad_add (&f2, &qone, &qr);
+	gnm_quad_add (&f2, &gnm_quad_one, &qr);
 	gnm_quad_sqrt (&f2, &f2);
 	if (debug) g_printerr ("f2=%.20g\n", gnm_quad_value (&f2));
 
@@ -657,18 +708,18 @@ static gnm_float
 pochhammer_naive (gnm_float x, int n)
 {
 	void *state = gnm_quad_start ();
-	GnmQuad qp, qx, qone;
+	GnmQuad qp, qx;
 	gnm_float r;
 
-	gnm_quad_init (&qone, 1);
-	qp = qone;
+	qp = gnm_quad_one;
 	gnm_quad_init (&qx, x);
 	while (n-- > 0) {
 		gnm_quad_mul (&qp, &qp, &qx);
-		gnm_quad_add (&qx, &qx, &qone);
+		gnm_quad_add (&qx, &qx, &gnm_quad_one);
 	}
 	r = gnm_quad_value (&qp);
 	gnm_quad_end (state);
+
 	return r;
 }
 
@@ -681,7 +732,7 @@ pochhammer_naive (gnm_float x, int n)
  */
 
 gnm_float
-pochhammer (gnm_float x, gnm_float n, gboolean give_log)
+pochhammer (gnm_float x, gnm_float n)
 {
 	gnm_float rn, rx, lr;
 	GnmQuad m1, m2;
@@ -690,12 +741,8 @@ pochhammer (gnm_float x, gnm_float n, gboolean give_log)
 	if (gnm_isnan (x) || gnm_isnan (n))
 		return gnm_nan;
 
-	/* This isn't a fundamental restriction, but one we impose.  */
-	if (x <= 0 || x + n <= 0)
-		return gnm_nan;
-
 	if (n == 0)
-		return give_log ? 0 : 1;
+		return 1;
 
 	rx = gnm_floor (x + 0.5);
 	rn = gnm_floor (n + 0.5);
@@ -705,10 +752,8 @@ pochhammer (gnm_float x, gnm_float n, gboolean give_log)
 	 * We don't want to use this if x is also an integer
 	 * (but we might do so below if x is insanely large).
 	 */
-	if (n == rn && x != rx && n >= 0 && n < 40) {
-		gnm_float r = pochhammer_naive (x, (int)n);
-		return give_log ? gnm_log (r) : r;
-	}
+	if (n == rn && x != rx && n >= 0 && n < 40)
+		return pochhammer_naive (x, (int)n);
 
 	if (!qfactf (x + n - 1, &m1, &e1) &&
 	    !qfactf (x - 1, &m2, &e2)) {
@@ -721,9 +766,19 @@ pochhammer (gnm_float x, gnm_float n, gboolean give_log)
 		r = gnm_quad_value (&qr);
 		gnm_quad_end (state);
 
-		return give_log
-			? gnm_log (r) + M_LN2gnum * de
-			: gnm_ldexp (r, de);
+		return gnm_ldexp (r, de);
+	}
+
+	if (x == rx && x <= 0) {
+		if (n != rn)
+			return 0;
+		if (x == 0)
+			return (n > 0)
+				? 0
+				: ((gnm_fmod (-n, 2) == 0 ? +1 : -1) /
+				   gnm_fact (-n));
+		if (n > -x)
+			return gnm_nan;
 	}
 
 	/*
@@ -731,23 +786,14 @@ pochhammer (gnm_float x, gnm_float n, gboolean give_log)
 	 * insanely big, possibly both.
 	 */
 
-	if (gnm_abs (x) < 1) {
-		/* n is big. */
-		if (give_log)
-			goto via_log;
-		else
-			return gnm_pinf;
-	}
+	if (gnm_abs (x) < 1)
+		return gnm_pinf;
 
-	if (n < 0) {
-		gnm_float r = pochhammer (x + n, -n, give_log);
-		return give_log ? -r : 1 / r;
-	}
+	if (n < 0)
+		return 1 / pochhammer (x + n, -n);
 
-	if (n == rn && n >= 0 && n < 100) {
-		gnm_float r = pochhammer_naive (x, (int)n);
-		return give_log ? gnm_log (r) : r;
-	}
+	if (n == rn && n >= 0 && n < 100)
+		return pochhammer_naive (x, (int)n);
 
 	if (gnm_abs (n) < 1) {
 		/* x is big.  */
@@ -757,17 +803,16 @@ pochhammer (gnm_float x, gnm_float n, gboolean give_log)
 		pochhammer_small_n (x, n, &qr);
 		r = gnm_quad_value (&qr);
 		gnm_quad_end (state);
-		return give_log ? gnm_log (r) : r;
+		return r;
 	}
 
 	/* Panic mode.  */
 	g_printerr ("x=%.20g  n=%.20g\n", x, n);
-via_log:
 	lr = ((x - 0.5) * gnm_log1p (n / x) +
 	      n * gnm_log (x + n) -
 	      n +
 	      (lgammacor (x + n) - lgammacor (x)));
-	return give_log ? lr : gnm_exp (lr);
+	return gnm_exp (lr);
 }
 
 /* ------------------------------------------------------------------------- */
@@ -858,7 +903,8 @@ qfactf (gnm_float x, GnmQuad *mant, int *exp2)
 		else {
 			GnmQuad b;
 
-			gnm_quad_init (&b, -gnm_sinpi (x)); /* ? */
+			gnm_quad_init (&b, -x);
+			gnm_quad_sinpi (&b, &b);
 			gnm_quad_mul (&b, &b, mant);
 			gnm_quad_div (mant, &gnm_quad_pi, &b);
 			*exp2 = -*exp2;
@@ -880,8 +926,7 @@ qfactf (gnm_float x, GnmQuad *mant, int *exp2)
 		*exp2 = 0;
 
 		/* sqrt(2Pi) */
-		gnm_quad_add (&f1, &gnm_quad_pi, &gnm_quad_pi);
-		gnm_quad_sqrt (&f1, &f1);
+		gnm_quad_sqrt (&f1, &gnm_quad_2pi);
 		if (debug) g_printerr ("f1=%.20g\n", gnm_quad_value (&f1));
 
 		/* (y/e)^y */
@@ -907,7 +952,7 @@ qfactf (gnm_float x, GnmQuad *mant, int *exp2)
 
 		if (debug) g_printerr ("G(x+1)=%.20g * 2^%d %s\n", gnm_quad_value (mant), *exp2, res ? "overflow" : "");
 	} else {
-		GnmQuad s, mFw;
+		GnmQuad s, qx, mFw;
 		gnm_float w, f;
 		int eFw;
 
@@ -919,14 +964,13 @@ qfactf (gnm_float x, GnmQuad *mant, int *exp2)
 		 */
 		w = gnm_floor (x + 0.5);
 		f = x - w;
+		gnm_quad_init (&qx, x);
 
 		gnm_quad_init (&s, 1);
-		while (x < 20) {
-			GnmQuad a;
-			x++;
+		while (w < 20) {
+			gnm_quad_add (&qx, &qx, &gnm_quad_one);
 			w++;
-			gnm_quad_init (&a, x);
-			gnm_quad_mul (&s, &s, &a);
+			gnm_quad_mul (&s, &s, &qx);
 		}
 
 		if (qfacti ((int)w, &mFw, &eFw)) {
@@ -948,6 +992,27 @@ qfactf (gnm_float x, GnmQuad *mant, int *exp2)
 	return res;
 }
 
+/* 0: ok, 1: overflow, 2: nan */
+static int
+qgammaf (gnm_float x, GnmQuad *mant, int *exp2)
+{
+	if (x < -1.5 || x > 0.5)
+		return qfactf (x - 1, mant, exp2);
+	else if (gnm_isnan (x) || x == 0)
+		return 2;
+	else {
+		void *state = gnm_quad_start ();
+		GnmQuad qx;
+
+		qfactf (x, mant, exp2);
+		gnm_quad_init (&qx, x);
+		gnm_quad_div (mant, mant, &qx);
+		rescale_mant_exp (mant, exp2);
+		gnm_quad_end (state);
+		return 0;
+	}
+}
+
 /* ------------------------------------------------------------------------- */
 
 gnm_float
@@ -981,7 +1046,7 @@ combin (gnm_float n, gnm_float k)
 		return c;
 	}
 
-	if (k < 30) {
+	if (k < 100) {
 		void *state = gnm_quad_start ();
 		GnmQuad p, a, b;
 		gnm_float c;
@@ -1001,11 +1066,7 @@ combin (gnm_float n, gnm_float k)
 		return c;
 	}
 
-	{
-		gnm_float lp = pochhammer (n - k + 1, k, TRUE) -
-			gnm_lgamma (k + 1);
-		return gnm_floor (0.5 + gnm_exp (lp));
-	}
+	return pochhammer (n - k + 1, k) / gnm_fact (k);
 }
 
 gnm_float
@@ -1014,7 +1075,7 @@ permut (gnm_float n, gnm_float k)
 	if (k < 0 || k > n || n != gnm_floor (n) || k != gnm_floor (k))
 		return gnm_nan;
 
-	return pochhammer (n - k + 1, k, FALSE);
+	return pochhammer (n - k + 1, k);
 }
 
 /* ------------------------------------------------------------------------- */
@@ -1063,3 +1124,278 @@ lgamma_r (double x, int *signp)
 #endif
 
 /* ------------------------------------------------------------------------- */
+
+
+static const gnm_float lanczos_g = GNM_const (808618867.0) / 134217728;
+
+/*
+ * This Mathematica sniplet computes the Lanczos gamma coefficients:
+ *
+ * Dr[k_]:=DiagonalMatrix[Join[{1},Array[-Binomial[2*#-1,#]*#&,k]]]
+ * c[k_]:= Array[If[#1+#2==2,1/2,If[#1>=#2,(-1)^(#1+#2)*4^(#2-1)*(#1-1)*(#1+#2-3)!/(#1-#2)!/(2*#2-2)!,0]]&,{k+1,k+1}]
+ * Dc[k_]:=DiagonalMatrix[Array[2*(2*#-3)!!&,k+1]]
+ * B[k_]:=Array[If[#1==1,1,If[#1<=#2,(-1)^(#2-#1)*Binomial[#1+#2-3,#1+#1-3],0]]&,{k+1,k+1}]
+ * M[k_]:=(Dr[k].B[k]).(c[k].Dc[k])
+ * f[g_,k_]:=Array[Sqrt[2]*(E/(2*(#-1+g)+1))^(#-1/2)&,k+1]
+ * a[g_,k_]:=M[k].f[g,k]*Exp[g]/Sqrt[2*Pi]
+ *
+ * The result of a[g,k] will contain both positive and negative constants.
+ * Most people using the Lanczos series do not understand that a naive
+ * implemetation will suffer significant cancellation errors.  The error
+ * estimates assume the series is computed without loss!
+ *
+ * Following Boost we multiply the entire partial fraction by Pochhammer[z+1,k]
+ * That gives us a polynomium with positive coefficient.  For kicks we toss
+ * the constant factor back in.
+ *
+ * b[g_,k_]:=Sum[a[g,k][[i+1]]/If[i==0,1,(z+i)],{i,0,k}]*Pochhammer[z+1,k]
+ * c13b:=Block[{$MaxExtraPrecision=500},FullSimplify[N[b[808618867/134217728,12]*Sqrt[2*Pi]/Exp[808618867/134217728],300]]]
+ *
+ * Finally we recast that in terms of gamma's argument:
+ *
+ * N[CoefficientList[c13b /. z->(zp-1),{zp}],50]
+ *
+ * Enter complex numbers, exit simplicity.  The error bounds for the
+ * Lanczos approximation are bounds for the absolute value of the result.
+ * The relative error on one of the coordinates can be much higher.
+ */
+static const gnm_float lanczos_num[] = {
+	GNM_const(56906521.913471563880907910335591226868592353221448),
+	GNM_const(103794043.11634454519062710536160702385539539810110),
+	GNM_const(86363131.288138591455469272889778684223419113014358),
+	GNM_const(43338889.324676138347737237405905333160850993321475),
+	GNM_const(14605578.087685068084141699827913592185707234229516),
+	GNM_const(3481712.1549806459088207101896477455646802362321652),
+	GNM_const(601859.61716810987866702265336993523025071425740828),
+	GNM_const(75999.293040145426498753034435989091370919973262979),
+	GNM_const(6955.9996025153761403563101155151989875259157712039),
+	GNM_const(449.94455690631681194468586076509884096232715968614),
+	GNM_const(19.519927882476174828478609662356521362076846583112),
+	GNM_const(0.50984166556566761881251786448046945099926051133936),
+	GNM_const(0.0060618423462489065257837539645559368832224636654970)
+};
+
+/* CoefficientList[Pochhammer[z,12],z] */
+static const guint32 lanczos_denom[G_N_ELEMENTS(lanczos_num)] = {
+	0, 39916800, 120543840, 150917976, 105258076, 45995730,
+	13339535, 2637558, 357423, 32670, 1925, 66, 1
+};
+
+void
+complex_gamma (complex_t *dst, complex_t const *src)
+{
+	if (complex_real_p (src)) {
+		complex_init (dst, gnm_gamma (src->re), 0);
+	} else if (src->re < 0) {
+		/* Gamma(z) = pi / (sin(pi*z) * Gamma(-z+1)) */
+		complex_t a, b, mz;
+
+		complex_init (&mz, -src->re, -src->im);
+		complex_fact (&a, &mz);
+
+		complex_init (&b,
+			      M_PIgnum * gnm_fmod (src->re, 2),
+			      M_PIgnum * src->im);
+		/* Hmm... sin overflows when b.im is large.  */
+		complex_sin (&b, &b);
+
+		complex_mul (&a, &a, &b);
+
+		complex_init (&b, M_PIgnum, 0);
+
+		complex_div (dst, &b, &a);
+	} else {
+		complex_t zmh, zmhd2, zmhpg, f, f2, p, q, pq;
+		int i;
+
+		i = G_N_ELEMENTS(lanczos_num) - 1;
+		complex_init (&p, lanczos_num[i], 0);
+		complex_init (&q, lanczos_denom[i], 0);
+		while (--i >= 0) {
+			complex_mul (&p, &p, src);
+			p.re += lanczos_num[i];
+			complex_mul (&q, &q, src);
+			q.re += lanczos_denom[i];
+		}
+		complex_div (&pq, &p, &q);
+
+		complex_init (&zmh, src->re - 0.5, src->im);
+		complex_init (&zmhpg, zmh.re + lanczos_g, zmh.im);
+		complex_init (&zmhd2, zmh.re * 0.5, zmh.im * 0.5);
+		complex_pow (&f, &zmhpg, &zmhd2);
+
+		zmh.re = -zmh.re; zmh.im = -zmh.im;
+		complex_exp (&f2, &zmh);
+		complex_mul (&f2, &f, &f2);
+		complex_mul (&f2, &f2, &f);
+
+		complex_mul (dst, &f2, &pq);
+	}
+}
+
+/* ------------------------------------------------------------------------- */
+
+void
+complex_fact (complex_t *dst, complex_t const *src)
+{
+	if (complex_real_p (src)) {
+		complex_init (dst, gnm_fact (src->re), 0);
+	} else {
+		/*
+		 * This formula is valid for all arguments except zero
+		 * which we conveniently handled above.
+		 */
+		complex_t gz;
+		complex_gamma (&gz, src);
+		complex_mul (dst, &gz, src);
+	}
+}
+
+/* ------------------------------------------------------------------------- */
+
+static void
+igamma_cf (complex_t *dst, const complex_t *a, const complex_t *z)
+{
+	complex_t A0, A1, B0, B1;
+	int i;
+	const gboolean debug_cf = FALSE;
+
+	complex_init (&A0, 1, 0);
+	complex_init (&A1, 0, 0);
+	complex_init (&B0, 0, 0);
+	complex_init (&B1, 1, 0);
+
+	for (i = 1; i < 100; i++) {
+		complex_t ai, bi, t1, t2, c1, c2, A2, B2;
+		gnm_float m;
+		const gnm_float BIG = GNM_const(18446744073709551616.0);
+
+		if (i == 1)
+			complex_init (&ai, 1, 0);
+		else if (i & 1) {
+			gnm_float f = (i >> 1);
+			complex_init (&ai, z->re * f, z->im * f);
+		} else {
+			complex_t f;
+			complex_init (&f, -(a->re + ((i >> 1) - 1)), -a->im);
+			complex_mul (&ai, &f, z);
+		}
+		complex_init (&bi, a->re + (i - 1), a->im);
+
+		/* Update A. */
+		complex_mul (&t1, &bi, &A1);
+		complex_mul (&t2, &ai, &A0);
+		complex_add (&A2, &t1, &t2);
+		A0 = A1; A1 = A2;
+
+		/* Update B. */
+		complex_mul (&t1, &bi, &B1);
+		complex_mul (&t2, &ai, &B0);
+		complex_add (&B2, &t1, &t2);
+		B0 = B1; B1 = B2;
+
+		/* Rescale */
+		m = gnm_abs (B1.re) + gnm_abs (B1.im);
+		if (m >= BIG || m <= 1 / BIG) {
+			int e;
+			gnm_float s;
+			(void)frexp (m, &e);
+			s = ldexp (1, -e);
+			A0.re *= s; A0.im *= s;
+			A1.re *= s; A1.im *= s;
+			B0.re *= s; B0.im *= s;
+			B1.re *= s; B1.im *= s;
+			if (debug_cf)
+				g_printerr ("rescale\n");
+		}
+
+		/* Check for convergence */
+		complex_mul (&t1, &A1, &B0);
+		complex_mul (&t2, &A0, &B1);
+		complex_sub (&c1, &t1, &t2);
+
+		complex_mul (&c2, &B0, &B1);
+
+		complex_div (&t1, &A1, &B1);
+		if (debug_cf) {
+			g_printerr ("  a : %.20g + %.20g I\n", ai.re, ai.im);
+			g_printerr ("  b : %.20g + %.20g I\n", bi.re, bi.im);
+			g_printerr ("  A : %.20g + %.20g I\n", A1.re, A1.im);
+			g_printerr ("  B : %.20g + %.20g I\n", B1.re, B1.im);
+			g_printerr ("%3d : %.20g + %.20g I\n", i, t1.re, t1.im);
+		}
+
+		if (complex_mod (&c1) <= complex_mod (&c2) * (16 * GNM_EPSILON))
+			break;
+	}
+
+	if (i == 100) {
+		/* Make the failure obvious. */
+		dst->re = dst->im = gnm_nan;
+		g_printerr ("igamma_cf not converged\n");
+		return;
+	}
+
+	complex_div (dst, &A1, &B1);
+}
+
+
+void
+complex_igamma (complex_t *dst, const complex_t *a, const complex_t *z,
+		gboolean lower, gboolean regularized)
+{
+	complex_t res, f, mz;
+
+	if (complex_zero_p (a)) {
+		if (!lower && !regularized)
+			complex_gamma (dst, z);
+		else
+			complex_init (dst, lower ? 0 : 1, 0);
+		return;
+	}
+
+	if (complex_real_p (a) && a->re >= 0 &&
+	    complex_real_p (z) && z->re >= 0) {
+		complex_init (&res, pgamma (z->re, a->re, 1, lower, FALSE), 0);
+		if (!regularized) {
+			complex_t g;
+			complex_gamma (&g, a);
+			complex_mul (&res, &res, &g);
+		}
+		*dst = res;
+		return;
+	}
+
+	igamma_cf (&res, a, z);
+
+	/*
+	 * FIXME: The following three blocks should be handled without
+	 * creating big numbers.
+	 */
+
+       	mz.re = -z->re, mz.im = -z->im;
+	complex_exp (&f, &mz);
+	complex_mul (&res, &res, &f);
+	complex_pow (&f, z, a);
+	complex_mul (&res, &res, &f);
+
+	if (!regularized && lower) {
+		/* Nothing */
+	} else {
+		complex_t g;
+		complex_gamma (&g, a);
+
+		if (regularized) {
+			complex_div (&res, &res, &g);
+			if (!lower)
+				res.re = 1 - res.re;
+		} else {
+			/* !lower here */
+			complex_sub (&res, &g, &res);
+		}
+	}
+
+	*dst = res;
+}
+
+/* ------------------------------------------------------------------------- */
